@@ -195,3 +195,102 @@ mlogit.optim <- function(f, start,
                  )
   result
 }
+
+
+
+mlogit.optim <- function(f, start,
+                         method = c('bfgs', 'nr', 'bhhh'),
+                         iterlim = 2000,
+                         tol = 1E-06,
+                         print.level = 1,
+                         constPar = NULL,
+                         ...){
+  # construct a call for the function
+  param <- start
+  method <- match.arg(method)
+  callT <- match.call(expand.dots = TRUE)
+  f <- callT
+  optimoptions <- c('iterlim', 'tol', 'method', 'print.level', 'constPar')
+  m <- match(optimoptions, names(callT), 0L)
+  if (sum(m)) f <- f[-m]
+  f[[1]] <- as.name(f[[2]])
+  K <- length(param)
+  fixed <- rep(FALSE, K)
+  if (!is.null(constPar)) fixed[constPar] <- TRUE
+  f$gradient <- TRUE
+  if (method == 'nr') f$hessian <- TRUE else f$hessian <- FALSE
+  f[[2]] <- NULL
+  names(f)[2] <- 'param'
+  chi2 <- 1E+10
+  i <- 0
+  # eval a first time the function, the gradient and the hessian
+
+###################################
+##  Test the gradient
+##   nd <- f
+##   nd[["f"]] <- nd[[1]]
+##   nd[[1]] <- as.name("numderiv")
+##   x <- eval(nd, parent.frame())
+##   print(x)
+###################################
+
+  x <- eval(f, parent.frame())
+  if (print.level > 0)
+    cat(paste("Initial value of the function :", as.numeric(x), "\n"))
+  g <- attr(x, "gradient")
+  if (method == 'nr')   H <- attr(x, "hessian")[!fixed, !fixed]
+  if (method == 'bhhh') H <- crossprod(attr(x, "gradi")[, !fixed])
+  if (method == 'bfgs') Hm1 <- solve(crossprod(attr(x, "gradi")[, !fixed]))
+  d <- rep(0, K)
+  while(abs(chi2) > tol && i < iterlim){
+    if (method == "bfgs") d[!fixed] <- - as.vector(Hm1 %*% g[!fixed])
+    else d[!fixed] <- - as.vector(solve(H, g[!fixed]))
+    i <- i + 1
+    oldx <- x
+    oldg <- g
+    f$param <- param
+    f$direction <- d
+    f$initial.value <- x
+    x <- eval(f, parent.frame())
+    g <- attr(x, "gradient")
+    step <- attr(x, "step")
+    param[!fixed] <- param[!fixed] + step * d[!fixed]
+    if (method == 'nr')   H <- attr(x, "hessian")[!fixed, !fixed]
+    if (method == 'bhhh') H <-  crossprod(attr(x, "gradi")[, !fixed])
+    if (method == 'bfgs'){
+      incr <- step * d
+      y <- g - oldg
+      Hm1 <- Hm1 +
+        outer( incr[!fixed], incr[!fixed]) *
+          (sum(y[!fixed] * incr[!fixed]) +
+           as.vector( t(y[!fixed]) %*% Hm1 %*% y[!fixed])) / sum(incr[!fixed] * y[!fixed])^2 -
+             (Hm1 %*% outer(y[!fixed], incr[!fixed])
+              + outer(incr[!fixed], y[!fixed]) %*% Hm1)/
+                sum(incr[!fixed] * y[!fixed])
+    }
+    chi2 <- -  crossprod(d[!fixed], oldg[!fixed])
+    if (print.level > 0){
+      chaine <- paste("iteration ",i,", step = ",step,
+                      ", lnL = ",round(x,8),", chi2 = ",
+                      round(chi2,8),"\n",sep="")
+    cat(chaine)
+    }
+    if (print.level > 1){
+      resdet <- rbind(param = param, gradient = g)
+      print(round(resdet,3))
+      cat("--------------------------------------------\n")
+    }
+  }
+  if (i >= iterlim) message = "maximum number of iterations reached"
+  else message = "optimum reached"
+  if (method == 'bfgs') H <- solve(Hm1)
+  names(attr(x, 'gradient')) <- colnames(attr(x, 'gradi')) <- names(param)
+  attr(x, "fixed") <- fixed
+  est.stat = structure(list(elaps.time = NULL, nb.iter = i, eps = chi2,
+    method = method, message = message), class = 'est.stat')
+  result <- list(optimum = x,
+                 coefficients = param,
+                 est.stat = est.stat
+                 )
+  result
+}
